import os


import pytest


def test_system_discovery_service():
    from gbfs.const import package_tests_fixtures_dirpath
    from gbfs.providers import systems_provider_local_csv
    from gbfs.services import SystemDiscoveryService
    from gbfs.data.fetchers import LocalJSONFetcher

    ds = SystemDiscoveryService(systems_provider=systems_provider_local_csv)

    assert ds.systems
    assert ds.system_ids
    assert len(ds.systems) == len(ds.system_ids)
    assert ds.get_system_by_id('ABU')
    assert ds._instantiate_client(os.path.join(package_tests_fixtures_dirpath, 'gbfs.json'), 'en',
                                  json_fetcher=LocalJSONFetcher())



import pytest

def test_client():
    from gbfs.client import GBFSClient


def test_system_discovery_service():
    from gbfs.services import SystemDiscoveryService
    ds = SystemDiscoveryService()
    assert ds.system_ids



import os


package_root_dirpath = os.path.abspath(os.path.dirname(__file__))

package_static_dirname = 'static'
package_static_dirpath = os.path.join(package_root_dirpath, package_static_dirname)

package_tests_dirname = 'tests'
package_tests_dirpath = os.path.join(package_root_dirpath, package_tests_dirname)

package_tests_fixtures_dirname = 'fixtures'
package_tests_fixtures_dirpath = os.path.join(package_tests_dirpath, package_tests_fixtures_dirname)

gbfs_systems_csv_remote_url = 'https://raw.githubusercontent.com/NABSA/gbfs/master/systems.csv'
gbfs_systems_csv_local_filepath = os.path.join(package_static_dirpath, 'systems.csv')


class gbfs_systems_csv_fields:
    country_code       = 'Country Code'
    name               = 'Name'
    location           = 'Location'
    system_id          = 'System ID'
    url                = 'URL'
    auto_discovery_url = 'Auto-Discovery URL'


gbfs_client_default_language = 'en'



import os


import pytest


# TODO: Use mock network services
def test_gbfs_client():
    from gbfs.const import package_tests_fixtures_dirpath
    from gbfs.data.fetchers import LocalJSONFetcher
    from gbfs.client import GBFSClient

    c = GBFSClient(os.path.join(package_tests_fixtures_dirpath, 'gbfs.json'), 'en',
                   json_fetcher=LocalJSONFetcher())

    assert c.feeds
    assert c.feed_names
    #assert c.request_feed('station_status')


from gbfs.const import gbfs_systems_csv_local_filepath, gbfs_systems_csv_remote_url
from gbfs.data.fetchers import RemoteCSVFetcher, LocalCSVFetcher
from gbfs.data.providers import SystemDataProvider

__all__ = [
    'systems_provider_local_csv',
    'systems_provider_remote_csv',
    'systems_provider_default'
]

local_csv_fetcher = LocalCSVFetcher()
remote_csv_fetcher = RemoteCSVFetcher()

systems_provider_local_csv = SystemDataProvider(local_csv_fetcher, gbfs_systems_csv_local_filepath)
systems_provider_remote_csv = SystemDataProvider(remote_csv_fetcher, gbfs_systems_csv_remote_url)

systems_provider_default = systems_provider_remote_csv


from gbfs.providers import systems_provider_remote_csv
from gbfs.client import GBFSClient
from gbfs.const import gbfs_systems_csv_fields


__all__ = ['SystemDiscoveryService']


class SystemDiscoveryService(object):
    """GBFS client discovery service"""

    _default_language = 'en'
    _client_cls = None
    _systems_provider = None
    _system_attrs = None

    def __init__(self, run_on_init=True, systems_provider=None):
        if systems_provider:
            self._systems_provider = systems_provider

        assert self._client_cls
        assert self._systems_provider
        assert self._system_attrs

        self._systems_cache = {}

        if run_on_init:
            self._get_and_cache_all_systems()

    def _get_and_cache_all_systems(self):
        try:
            systems = self._systems_provider.get_all()
        except:
            raise

        for system in systems:
            system_id = system.get(self._system_attrs.system_id)
            if system_id is None:
                raise RuntimeError('Unexpected systems data format.')
            self._systems_cache[system_id] = system

    @property
    def systems(self):
        return list(self._systems_cache.values())

    @property
    def system_ids(self):
        if self._systems_cache:
            return list(self._systems_cache.keys())

    def get_system_by_id(self, system_id):
        return self._systems_cache.get(system_id)

    def instantiate_client(self, system_id, language=None):
        system = self._systems_cache.get(system_id)
        if system:
            system_url = system.get(self._system_attrs.auto_discovery_url)
            if system_url:
                return self._instantiate_client(system_url, language if language else self._default_language)

    def _instantiate_client(self, system_url, language, json_fetcher=None):
        return self._client_cls(system_url, language, json_fetcher=json_fetcher)

# Runtime config
SystemDiscoveryService._system_attrs = gbfs_systems_csv_fields
SystemDiscoveryService._client_cls = GBFSClient
SystemDiscoveryService._systems_provider = systems_provider_remote_csv



import datetime


from gbfs.services import SystemDiscoveryService


def example():
    ds = SystemDiscoveryService()

    # Count number of systems discovered
    print('Found {} bikeshare systems.'.format(len(ds.systems)))

    # Show 10 systems...
    print('Showing 10 of the systems_ids found:')
    for system_id in ds.system_ids[:10]:
        print(system_id)
    print('')

    # Select a system with a known system_id
    citibike_system_id = 'NYC'
    print('Showing system information for a known station_id: {}'.format(citibike_system_id))
    system = ds.get_system_by_id(citibike_system_id)
    print(system)
    print('')

    # Instantiate a new GBFSClient from known system_id
    print('Instantiating a GBFSClient for known system_id: {}'.format(citibike_system_id))
    client = ds.instantiate_client(citibike_system_id)
    print('')

    # A client can be instantiated directly, without using the SystemDiscoveryService,
    # if its Auto-Discovery URL is already known:
    from gbfs.client import GBFSClient
    client2 = GBFSClient('https://gbfs.citibikenyc.com/gbfs/gbfs.json', 'en')

    # Show the names of available feeds from client
    print('{} system publishes {} feeds:'.format(citibike_system_id, len(client.feeds)))
    for feed_name in client.feed_names:
        print(feed_name)
    print('')

    # Request some feeds and put together a useful live stat for some arbitrary station

    print('Requesting some live feeds...')
    station_information = client.request_feed('station_information')
    station_status = client.request_feed('station_status')

    stations = station_information.get('data').get('stations')
    print('{} system has {} stations in its network'.format(citibike_system_id, len(stations)))
    print('')

    first_station = stations[0]
    
    statuses = station_status.get('data').get('stations')
    first_station_status = [s for s in statuses if s.get('station_id') == first_station.get('station_id')][0]

    staleness = datetime.datetime.utcnow() - station_status.get('last_updated')

    print('Station {} has a total capacity of {} and (as of {} seconds ago) has {} bikes available for rent.'.format(
        first_station.get('name'), first_station.get('capacity'), staleness.seconds,
        first_station_status.get('num_bikes_available')))
    print('')



if __name__ == '__main__':
    example()


import csv
import datetime

import csv23
import six
import abc

@six.add_metaclass(abc.ABCMeta)
class DataProvider(object):
    @abc.abstractmethod
    def get_all(self):
        pass


class SystemDataProvider(DataProvider):
    _csv_dict_reader = csv23.DictReader

    def __init__(self, file_fetcher, csv_url):
        self._file_fetcher = file_fetcher
        self._csv_url = csv_url
        assert self._csv_dict_reader

    def get_all(self):
        data = self._file_fetcher.fetch(self._csv_url)
        reader = self._csv_dict_reader(data)
        for item in reader:
            yield dict(item)


class StationDataProvider(DataProvider):
    _posix_to_datetime_func = datetime.datetime.utcfromtimestamp

    def __init__(self, json_fetcher, json_url):
        self._json_fetcher = json_fetcher
        self._json_url = json_url

    def get_all(self, json_url):
        data = self._json_fetcher.fetch(self._json_url)

        last_updated = data.get('last_updated')
        if last_updated:
            data['last_updated'] = self._posix_to_datetime_func(last_updated)

        return data



# See setup.py for humans: https://github.com/navdeep-G/setup.py

import io
import os
import sys
import shutil
from setuptools import setup, find_packages, Command

import versioneer


NAME = 'gbfs-client'
DESCRIPTION = 'Python client for discovering and capturing GBFS bikeshare feeds.'
URL = 'https://github.com/jakehadar/gbfs-client'
AUTHOR = 'Jake Hadar'
EMAIL = 'jake1025@gmail.com'
REQUIRES_PYTHON = '>=2.7'
VERSION = versioneer.get_version()


here = os.path.abspath(os.path.dirname(__file__))


def read_requirements(extension=None):
    ext = '' if extension is None else '-{}'.format(extension)
    filename = 'requirements{}.txt'.format(ext)
    with io.open(filename, encoding='utf-8') as f:
        requirements = [r.strip() for r in f.readlines()]
        return requirements


class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = [
        ('test', 't', 'Upload to PyPI test instance.')
    ]

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        self.test = False
        self.upload_args = ''
        self.install_args = ''

    def finalize_options(self):
        if self.test:
            self.upload_args = '--repository-url https://test.pypi.org/legacy/'
            self.install_args = '--index-url https://test.pypi.org/simple/'

    def run(self):
        try:
            import twine
        except ImportError:
            self.status('Please install twine to use upload command.')
            self.status('  $ pip install twine')
            self.status('Aborting.')
            sys.exit(1)

        self.status('Cleaning build...')
        os.system('{0} setup.py clean --all'.format(sys.executable))

        try:
            self.status('Removing previous builds...')
            shutil.rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution...')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))
        
        if self.test:
            self.status('Uploading the package to PyPI (test instance) via Twine...')
            os.system('twine upload --repository-url https://test.pypi.org/legacy/ dist/*')

            self.status('Installation command:')
            self.status('pip install --index-url https://test.pypi.org/simple/ {0}'.format(NAME))
            sys.exit()

        if '+' in VERSION:
            commits_ahead = VERSION.split('+')[1].split('.')[0]
            self.status('Build is {0} commits ahead of master.'.format(commits_ahead))
            self.status('Aborting.')

            sys.exit(1)

        self.status('Uploading the package to PyPI via Twine...')
        os.system('twine upload dist/*')
            
        self.status('Installation command:')
        self.status('pip install {0}'.format(NAME))

        self.status('Pushing git tags...')
        os.system('git tag v{0}'.format(VERSION))
        os.system('git push --tags')

        sys.exit()


cmdclass = versioneer.get_cmdclass()
cmdclass.update({'upload': UploadCommand})


setup(
    name=NAME,
    version=VERSION,
    author=AUTHOR,
    author_email=EMAIL,
    description=DESCRIPTION,
    url=URL,
    python_requires=REQUIRES_PYTHON,
    long_description=io.open('README.md', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    cmdclass=cmdclass,
    include_package_data=True,
    package_data={'gbfs': ['gbfs/static/systems.csv']},
    keywords='gbfs bikeshare client',
    packages=find_packages(exclude=['tests']),
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python',
    ],
    install_requires=read_requirements(),
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    extras_require={
        'dev': read_requirements('dev'),
        'test': read_requirements('test')
    },
    # entry_points={
    #     'console_scripts': ['mycli=mymodule:cli']
    # }
)



import datetime
import requests


from gbfs.data.fetchers import RemoteJSONFetcher
from gbfs.const import gbfs_client_default_language


__all__ = ['GBFSClient']


class GBFSClient(object):
    _json_fetcher = RemoteJSONFetcher()
    _posix_to_datetime_func = datetime.datetime.utcfromtimestamp
    _default_language = gbfs_client_default_language

    def __init__(self, url, language=None, json_fetcher=None):
        language = language or self._default_language

        if json_fetcher:
            self._json_fetcher = json_fetcher

        assert self._json_fetcher

        r = self._json_fetcher.fetch(url)
        
        data = r.get('data')
        if data is None:
            raise Exception('GBFS missing required key path: "data"')
        
        languages = data.keys()
        if language not in languages:
            raise Exception('Language must be one of: {}'.format(','.join(languages)))

        self.language = language
        
        feeds = data[language].get('feeds')
        if feeds is None:
            raise Exception('GBFS missing required json path: "data.{}.feeds"'.format(language))

        self.auto_discovery_url = url

        self.feeds = dict(
            map(lambda feed: (feed.get('name'), feed.get('url')), feeds)
        )
        
    @property
    def feed_names(self):
        return list(self.feeds.keys())

    def request_feed(self, feed_name):
        url = self.feeds.get(feed_name)
        if url is None:
            raise Exception('Feed name must be one of: {}'.format(','.join(self.feed_names)))

        data = self._json_fetcher.fetch(url)

        last_updated = data.get('last_updated')
        if last_updated:
            data['last_updated'] = self._posix_to_datetime_func(last_updated)

        return data

    def __repr__(self):
        return "{}('{}', '{}')".format(self.__class__.__name__, self.auto_discovery_url, self.language)


import abc
import json

import io
import six
import requests


@six.add_metaclass(abc.ABCMeta)
class FileFetcher(object):
    @abc.abstractmethod
    def fetch(url):
        pass


class LocalCSVFetcher(FileFetcher):
    def fetch(self, url):
        with io.open(url, 'rt', encoding='utf-8') as f:
            data = f.readlines()
        return data


class RemoteCSVFetcher(FileFetcher):
    _requests_module = requests

    def __init__(self, requests_module=None):
        if requests_module:
            self._requests_module = requests_module

        assert self._requests_module

    def fetch(self, url):
        response = self._requests_module.get(url)
        if response.status_code != 200:
            raise RuntimeError('HTTPS request for {} failed with status code {}' \
                               .format(url, response.status_code))
        return list(response.iter_lines(decode_unicode=True))


class LocalJSONFetcher(FileFetcher):
    _json_module = json

    def __init__(self, json_module=None):
        if json_module:
            self._json_module = json_module
        assert self._json_module

    def fetch(self, url):
        with io.open(url, 'r', encoding='utf-8') as f:
            data = self._json_module.load(f)
        return data


class RemoteJSONFetcher(FileFetcher):
    _requests_module = requests

    def __init__(self, requests_module=None):
        if requests_module:
            self._requests_module = requests_module
        assert self._requests_module

    def fetch(self, url):
        response = self._requests_module.get(url)
        if response.status_code != 200:
            raise RuntimeError('HTTPS request for {} failed with status code {}' \
                               .format(url, response.status_code))
        return response.json()
